var maxConcurrentUploads = 1;
var concurrentUploads = 0;
var uploadTotal = 0;
var bytesUploaded = 0;
var filesUploaded = 0;
var fileQueue = [];
var dirQueue = [];
var errors = 0;
var currentUpload = null;

var selectedAll = false;

// getElementById shorthand
function $id(id) {
	return document.getElementById(id);
}

function newDir() {
	$id("newDirDialog").setAttribute("open", "");
	$id("newDirName").value = "";
}

function newDirCancel() {
	$id("newDirDialog").removeAttribute("open");
}

function createNewDir() {
	var newDirName = $id("newDirName").value;
	if (newDirName.trim() == "")
	{
		alert("Directory name should not be whitespace");
		return;
	}
	var regex = /[?:|*<>\\\/\0]/g;
	if (regex.test(newDirName)) {
		alert("Directory name contains forbidden characters");
		return;
	}
	newDirCancel();
	var url = location.pathname + encodeURIComponent(newDirName);
	var http = new XMLHttpRequest();
    http.open("MKCOL", url, true);
    http.onload = function(e) {
		if(http.status != 201) {
			alert("Can't create directory: " + http.responseText);
		}
		location.reload();
    }
    http.send();
}

function getCheckedBoxes(chkboxName, encode = true) {
	var checkboxes = document.getElementsByName(chkboxName);
	var checked = [];
	for (var i=0; i<checkboxes.length; i++) {
	    if (checkboxes[i].checked) {
	        var value = checkboxes[i].nextSibling.innerText
		    if (encode)
		        checked.push(encodeURIComponent(value));
		    else checked.push(value);
	    }
	}
	return checked;
}

function downloadZIP() {
    var items = getCheckedBoxes("checked");
    if (items.length < 1) {
        alert("Must select at least one item to add to ZIP");
        return;
    }
    var dir = location.pathname.substring(6); //Already encoded

    var params = "zip=" + dir + "//" + items.join("/");
    location.href = "/zip/archive.zip?"+params;
}

function remove() {
    var items = getCheckedBoxes("checked");
    if (items.length < 1) {
        alert("Must select at least one item to delete");
        return;
    }
    if (!window.confirm("Are you sure you want to delete these " + items.length + " items?\n" + getCheckedBoxes("checked", false).join(", ")))
        return;
    var dir = location.pathname.substring(6); //Already encoded

    var params = "dir=" + dir + "&items=" + items.join("/");
    var url = "/delete?"+params;
    var xhr = new XMLHttpRequest();
    xhr.open("GET", url, true);
    xhr.onload = function(e) {
        if (xhr.status != 200)
            alert(xhr.responseText);
        else location.reload();
    }
    xhr.send();
}

window.onbeforeunload = function() {
	if(concurrentUploads)
		return "Leaving the page will cancel your upload.";
	else return;
};

window.onload = function() {
    var fileUpload = document.getElementById("fileupload");
    fileUpload.addEventListener('change', function(e) {
        e.preventDefault();

        fileList = []; //files is of type FileList, not array
        for(var i = 0; i < fileUpload.files.length; i++) {
            fileList.push(fileUpload.files[i]);
            uploadTotal += fileUpload.files[i].size;
        }
        fileQueue = fileQueue.concat(fileList);
        processQueue();
    });
    /*var dirUpload = document.getElementById("dirupload");
    dirUpload.addEventListener('change', function(e) {
        console.log(e);
    });*/
    deselectAll();
}

function processQueue() {
    while (concurrentUploads < maxConcurrentUploads) {
        if (dirQueue.length > 0) {
            processDirQueue();
            continue;
        }
        if (fileQueue.length == 0)
            break;
        var f = fileQueue.shift();
        sendFile(f);
    }
    $id("uploaded-lbl").innerText = filesUploaded + " files uploaded,  " + fileQueue.length + " in queue";
}

function processDirQueue() {
    concurrentUploads++;
    var progressText = document.createElement("a");
    $id("upload-box").appendChild(progressText);

    var dir = dirQueue.shift();
    var url = window.location + encodeURIParts(dir.fullPath)
    progressText.innerText = "Creating directories, " + dirQueue.length + " remaining";

    var xhr = new XMLHttpRequest();
    xhr.open("MKCOL", url, true);
    xhr.onload = function(e) {
        if (xhr.status != 201 && xhr.status != 405) { //Success or dir exists
            progressText.innerText = "Failed to create directory " + dir.name;
            errors++;
        } else progressText.remove();
        concurrentUploads--;
        onFinishedUpload();
    }
    xhr.send();
    currentUpload = xhr;
}

function sendFile(file) {
    concurrentUploads++;

    var xhr = new XMLHttpRequest();
    var progresstext = document.createElement("a");
    var pbar = document.createElement("progress");
    $id("upload-box").appendChild(progresstext);
    $id("upload-box").appendChild(pbar);

    var path = file.name;
    if (file.hasOwnProperty("path"))
        path = file.path;
    xhr.open("PUT", window.location + encodeURIParts(path), true);
    xhr.setRequestHeader("X-Last-Modified", file.lastModified);
    xhr.overrideMimeType(file.type);

    pbar.max = file.size;
    var loadedB = 0;
    xhr.upload.addEventListener("progress", function(e) {
        bytesUploaded += e.loaded - loadedB;
        loadedB = e.loaded;
        pbar.value = e.loaded;
        updateUploadPercent();
        progresstext.innerText = file.name + " " + hfs(e.loaded) + "/" + hfs(e.total);
    }, false);

    xhr.onreadystatechange = function () {
        if(xhr.readyState === 4) {
            pbar.remove();
            if (xhr.status != 201) {
                progresstext.innerText = "File " + file.name + " failed to upload. Status: " + xhr.status + "\n" + xhr.responseText;
                errors++;
            } else {
                progresstext.remove();
                filesUploaded++;
            }
            //File completed
            xhr.onreadystatechange = null; //Don't call twice
            bytesUploaded += file.size - loadedB;
            updateUploadPercent();
            concurrentUploads--;
            onFinishedUpload();
        }
    };

    xhr.send(file);
    currentUpload = xhr;
}

function onFinishedUpload() {
    if (dirQueue.length + fileQueue.length + concurrentUploads == 0) {
        if (errors > 0) {
            alert(errors + " items were not uploaded. Please refresh the page and try again.");
            errors = 0;
        }
        else location.reload();
    }
    processQueue(); //From callback, so no recursion
}

function updateUploadPercent() {
    var totalPercent = bytesUploaded / uploadTotal * 100;
    $id("upload-lbl").innerText = "Upload (" + totalPercent.toFixed(2) + "%)";
}

function toggleDropdown() {
  $id("upload-box").classList.toggle("show");
}

function deselectAll() {
    selectedAll = true;
    selectAll(); //Toggle
}

function selectAll() {
    var newState = !selectedAll;
    var checkboxes = document.getElementsByName("checked");
    for (var i=0; i<checkboxes.length; i++) {
       checkboxes[i].checked = newState;
    }
    selectedAll = newState;
    $id("select-all").innerText = selectedAll ? "Deselect All" : "Select All";
}

function cancelCurrent() {
    currentUpload.abort();
}

function cancelAll() {
    fileQueue = [];
    dirQueue = [];
    cancelCurrent();
    concurrentUploads = 0;
    bytesUploaded = 1;
    uploadTotal = 1;
    updateUploadPercent();
}

function encodeURIParts(encode) {
    var u = encode.split("/");
    for (var i = 0; i < u.length; i++)
        u[i] = encodeURIComponent(u[i]);
    return u.join("/");
}

function hfs(bytes, dp=2) {
  if (Math.abs(bytes) < 1024) {
    return bytes + ' B';
  }

  const units = ['kB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'];
  let u = -1;
  const r = 10**dp;

  do {
    bytes /= 1024;
    ++u;
  } while (Math.round(Math.abs(bytes) * r) / r >= 1024 && u < units.length - 1);

  return bytes.toFixed(dp) + ' ' + units[u];
}