var previousDropHighlight;
var dragMoveInProgress = false;
var traverseEnded;

window.addEventListener('load', function() {
    var main = document.getElementsByClassName("main")[0];
    main.addEventListener('drop', async function(ev) {
        ev.preventDefault();

        if (ev.dataTransfer.getData("text")) {
            dragMove(ev);
            return;
        }

        $id("upload-box").classList.add("show");
        document.getElementsByClassName("main")[0].id = "";

        fileList = [];
        traverseEnded = false;
        if (ev.dataTransfer.items) {
            for (var i = 0; i < ev.dataTransfer.items.length; i++) {
                if (ev.dataTransfer.items[i].kind === 'file') {
                    await traverseDirTree(ev.dataTransfer.items[i].webkitGetAsEntry());
                }
            }
        } else {
            for (var i = 0; i < ev.dataTransfer.files.length; i++) {
                fileList.push(ev.dataTransfer.files[i]);
                uploadTotal += ev.dataTransfer.files[i].size;
            }
        }

        fileQueue = fileQueue.concat(fileList);
        processQueue();
        traverseEnded = true;
    }, false);
    main.addEventListener('dragover', dragOverHandler, false);
    main.addEventListener('dragleave', dragOverHandler, false);
}, false);

function dragOverHandler(ev) {
    ev.preventDefault();
    if (!dragMoveInProgress)
        document.getElementsByClassName("main")[0].id = ev.type == "dragover" ? "dropzone" : "";
    else {
        var tgt = ev.target;
        if (!tgt.href)
            tgt = tgt.parentElement;
        if (tgt.parentElement.className.startsWith("item") && tgt.href.endsWith("/")) {
            if (previousDropHighlight)
                previousDropHighlight.id = "";
            tgt.parentElement.id = ev.type == "dragover" ? "dropzone" : "";
            previousDropHighlight = tgt.parentElement;
        }
    }
}

function dragStartHandler(ev) {
    //console.log(ev.target);
    ev.dataTransfer.setData("text", ev.target.href);
    dragMoveInProgress = true;
}

function dragEndHandler(ev) {
    dragMoveInProgress = false;
}

function dragMove(ev) {
    var source = ev.dataTransfer.getData("text");
    var target = ev.target.href;
    if(!target) {
        target = ev.target.parentElement.href;
        ev.target.parentElement.parentElement.id = "";
    } else ev.target.parentElement.id = "";
    if(!target) {
        console.warn("Could not identify where to move"); return;
    }
    if (!target.endsWith("/")) {
        console.log("Not a folder"); return;
    }
    if (source == target)
        return;
    if (source.endsWith("/"))
        source = source.substring(0, source.length - 1);
    var name = source.substring(source.lastIndexOf("/")+1);
    var xhr = new XMLHttpRequest();
    xhr.open("MOVE", source, true);
    xhr.setRequestHeader("Destination", target + name);
    xhr.onload = function(e) {
        if (xhr.status != 201)
            alert(xhr.responseText);
        else location.reload();
    }
    xhr.send();
}

async function traverseDirTree(entry) {
    console.log("Traversing");
    console.log(entry);
    if (entry.isFile) {
        console.log("<- Got file");
        entry.file((file) => {
            file.path = entry.fullPath;
            console.log("Adding full path: " + file.path);
            fileQueue.push(file);
            uploadTotal += file.size;
            if (traverseEnded)
                processQueue();
        });
    } else if (entry.isDirectory) {
        console.log("<- Got dir");
        dirQueue.push(entry);
        var reader = entry.createReader();
        var entries = await readAllDirectoryEntries(reader);
        for (var i = 0; i < entries.length; i++) {
            await traverseDirTree(entries[i]);
        }
    }
}

async function readAllDirectoryEntries(directoryReader) {
  let entries = [];
  let readEntries = await readEntriesPromise(directoryReader);
  while (readEntries.length > 0) {
    entries.push(...readEntries);
    readEntries = await readEntriesPromise(directoryReader);
  }
  return entries;
}

async function readEntriesPromise(directoryReader) {
  try {
    return await new Promise((resolve, reject) => {
      directoryReader.readEntries(resolve, reject);
    });
  } catch (err) {
    console.log(err);
  }
}

function getParams() {
    return new URLSearchParams(location.search);
}

function sortBy(by) {
    p = getParams();
    p.set("sortBy", by);
    if (p.has("sort"))
        p.set("sort", p.get("sort") == "desc" ? "asc" : "desc");
    else p.set("sort", "asc");
    reloadWithParams(p);
}

function reloadWithParams(p) {
    location.replace(location.pathname + "?" + p.toString());
}
